package arm_test

import (
	"testing"

	"git.urbach.dev/cli/q/src/arm"
	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/go/assert"
)

func TestMoveKeep(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   uint16
		Code     []byte
	}{
		{arm.X0, 0, []byte{0x00, 0x00, 0x80, 0xF2}},
		{arm.X0, 1, []byte{0x20, 0x00, 0x80, 0xF2}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("movk %s, %x", pattern.Register, pattern.Number)
		code := arm.MoveKeep(nil, pattern.Register, 0, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestMoveZero(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   uint16
		Code     []byte
	}{
		{arm.X0, 0, []byte{0x00, 0x00, 0x80, 0xD2}},
		{arm.X0, 1, []byte{0x20, 0x00, 0x80, 0xD2}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("movz %s, %x", pattern.Register, pattern.Number)
		code := arm.MoveZero(nil, pattern.Register, 0, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
