package config

import "runtime"

var (
	ConstantFold   bool // Calculates the result of operations on constants at compile time.
	Dry            bool // Skips writing the executable to disk.
	ShowAssembly   bool // Shows assembly instructions at the end.
	ShowStatistics bool // Shows statistics at the end.
	TargetArch     Arch // Target architecture.
	TargetOS       OS   // Target platform.
)

// Reset resets the configuration to its default values.
func Reset() {
	ShowAssembly = false
	ShowStatistics = false
	Dry = false

	switch runtime.GOARCH {
	case "amd64":
		TargetArch = X86
	case "arm64":
		TargetArch = ARM
	default:
		TargetArch = UnknownArch
	}

	switch runtime.GOOS {
	case "linux":
		TargetOS = Linux
	case "darwin":
		TargetOS = Mac
	case "windows":
		TargetOS = Windows
	default:
		TargetOS = UnknownOS
	}

	Optimize(true)
}

// Optimize enables or disables all the optimizations at once.
func Optimize(enable bool) {
	ConstantFold = enable
}
